//  RFIDFuntionsList.m
//  Created by Robin on 2016. 5. 16..

#import "RFIDFuntionsList.h"
#import "ModelSettings.h"
#import "GlobalDefine.h"
#import "Indicator.h"
#import "Utils.h"
#import "imageAlert.h"
#import <AsReaderDockSDK/AsReaderDockSDK.h>
#import <ExternalAccessory/ExternalAccessory.h>

@interface RFIDFuntionsList()<AsReaderRFIDDeviceDelegate, AsReaderDeviceDelegate>
{
}

@property (weak, nonatomic) IBOutlet UILabel *olPowerLever;
@property (weak, nonatomic) IBOutlet UILabel *olStopCondition;
@property (weak, nonatomic) IBOutlet UILabel *olOnOffTime;
@property (weak, nonatomic) IBOutlet UILabel *olRFChannel;

@property (weak, nonatomic) IBOutlet UILabel *olThresholdValue;
@property (weak, nonatomic) IBOutlet UILabel *olModulation;


@property (weak, nonatomic) IBOutlet UISwitch *btnPAGainMode;

@property (weak, nonatomic) IBOutlet UISwitch *olSwitchRSSIOn;
@property (weak, nonatomic) IBOutlet UISwitch *olHopping;
@property (weak, nonatomic) IBOutlet UILabel *olAntiCollisionMode;
@property (weak, nonatomic) IBOutlet UITableViewCell *rssiCell;

@end


@implementation RFIDFuntionsList

#pragma mark - Lifecycle
//----------------------------------------------------------------
- (void)viewDidLoad
{
    [super viewDidLoad];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:YES];
    AsReaderDevice *device = [AsReaderRFIDDevice sharedInstance];
    device.delegateDevice = self;
    device.delegateRFID = self;
    

    
    self.olPowerLever.text = @"---";
    self.olStopCondition.text = @"---";
    self.olOnOffTime.text = @"---";
    self.olRFChannel.text = @"---";
    self.olModulation.text = @"---";
    
    BOOL isRSSIOn = [[NSUserDefaults standardUserDefaults] boolForKey:@"RSSIOn"];
    
    [self.olSwitchRSSIOn setOn:isRSSIOn];
    
    int nTagCount =(int)[[NSUserDefaults standardUserDefaults] integerForKey:@"RFIDScanTagCount"];
    int nScanTime =(int)[[NSUserDefaults standardUserDefaults] integerForKey:@"RFIDScanTagTime"];
    int nCycle =(int)[[NSUserDefaults standardUserDefaults] integerForKey:@"RFIDScanTagInventory"];
    
    dispatch_async(dispatch_get_main_queue(),^{
        self.olStopCondition.text = [NSString stringWithFormat:@"%d/%d/%d", nTagCount, nScanTime, nCycle];
    });
    
    
    //Power
    [self performSelector:@selector(onTickRFIDPowerFuntion) withObject:nil afterDelay:.2f];
    //Stop
    [self performSelector:@selector(onTickStopConditionFuntion) withObject:nil afterDelay:.4f];
    //OnOffTime
    [self performSelector:@selector(onTickOnOffTimeFuntion) withObject:nil afterDelay:.6f];
    //Channel
    [self performSelector:@selector(onTickOnChannelFuntion) withObject:nil afterDelay:.8f];
    //AntiCollison
    [self performSelector:@selector(onTickAnticollisionFuntion) withObject:nil afterDelay:1.0f];
    
    [self performSelector:@selector(onTickModulationFuntion) withObject:nil afterDelay:1.2f];
    
    AsReaderInfo *info = [AsReaderInfo sharedInstance];
    if (([info.deviceModelNumber isEqualToString:@"ASR-030D"] && [self isV1Device]) ||
        ([info.deviceModelNumber isEqualToString:@"ASR-0230D"] && [self isV1Device])) {
        [self.rssiCell setHidden:YES];
    } else {
        /* Request PA GAIN MODE & RSSI ThresHold for RED4S */
        float hwVer = [info.deviceHardware floatValue];

        /* HW Version 1.X.X  RED 4    2.X.X = RED4S */
        if(hwVer >= 2.0) {
            // RSSI Threshold
            [self performSelector:@selector(onTickThresholdFuntion)   withObject:nil afterDelay:1.4f];
        } else {
            [self.view setUserInteractionEnabled:YES];
        }
    }
}

- (BOOL)isV1Device {
    NSArray *accessories = [[EAAccessoryManager sharedAccessoryManager] connectedAccessories];
    for (EAAccessory *obj in accessories)
    {
        NSString *string = [obj hardwareRevision];
        string = [string stringByReplacingOccurrencesOfString:@" " withString:@""];
        if ([string hasPrefix:@"1"] || [string hasPrefix:@"0"]) {
            return YES;
        }
    }
    return NO;
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    AsReaderDevice *device = [AsReaderRFIDDevice sharedInstance];
    device.delegateDevice = nil;
    device.delegateRFID = nil;
}

#pragma mark - Private method
//----------------------------------------------------------------

- (void)onTickModulationFuntion
{
    [[AsReaderRFIDDevice sharedInstance]getModulation];
}

- (void)onTickThresholdFuntion
{
    [[AsReaderRFIDDevice sharedInstance] getRSSIThreshold];
}

- (void)onTickAnticollisionFuntion
{
    [[AsReaderRFIDDevice sharedInstance] getAnticollision];
}

- (void)onTickStopConditionFuntion
{
    [[AsReaderRFIDDevice sharedInstance] getStopCondition];
}

- (void)onTickRFIDPowerFuntion
{
    [[AsReaderRFIDDevice sharedInstance] getOutputPowerLevel];
}

- (void)onTickOnOffTimeFuntion
{
    [[AsReaderRFIDDevice sharedInstance] getFhLbtParameter];
}

- (void)onTickOnChannelFuntion
{
    [[AsReaderRFIDDevice sharedInstance] getChannel];
}

#pragma mark - IBAction
//----------------------------------------------------------------
- (IBAction)btnSaveRegistry:(id)sender
{
    //Sotre data into Registry
    [[AsReaderRFIDDevice sharedInstance] updateRegistry];
}

- (IBAction)actionRSSISwitch:(UISwitch *)sender
{
    UISwitch *sw = (UISwitch*)sender;
    [AsReaderDevice setReadRSSIMode:sw.isOn];
    [[NSUserDefaults standardUserDefaults] setBool:sw.isOn forKey:@"RSSIOn"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

#pragma mark - AsReaderRFIDDeviceDelegate
//----------------------------------------------------------------
- (void)rssiThresholdReceived:(uint16_t)threshold
{
    dispatch_async(dispatch_get_main_queue(),^{
        [self->_olThresholdValue setText:[NSString stringWithFormat:@"%d",threshold]];
    });
}

- (void)updatedRegistry:(int)status
{
    NSString *strResult = @"";
    
    //Sucess
    if(status == 0x00) {
        strResult = @"Successfully Stored Data into Registry. Please Reset Hardware.";
    } else {
        strResult = @"Fail Store into Registry";
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [Utils showAlert:@"Success" message:@"Update Registry Result" view:self];
    });
}


- (void)modulationParamReceived:(int)blf rxMod:(int)rxMod dr:(int)dr{

    
    NSString *blfStr= @"";
    NSString *rxModStr= @"";
    
    if(blf ==0x00A0){
        blfStr =@"160KHz";
    }else if(blf ==0x00FA){
        blfStr =@"250KHz";
    }else if(blf ==0x0140){
        blfStr =@"320KHz";
    }else if (blf ==0x0280){
        blfStr =@"640KHz";
    }
    
    if(rxMod == 0x00){
        rxModStr =  @"FM0";
    }else if(rxMod == 0x01){
        rxModStr =  @"M2";
    }else if(rxMod == 0x02){
        rxModStr =  @"M4";
    }else if(rxMod == 0x03){
        rxModStr =  @"M8";
    }
    
    
    NSString *currentModulation = [NSString stringWithFormat:@"%@, %@, DR=64/3",blfStr,rxModStr];
    NSLog(@"%@",currentModulation);
    
     
   dispatch_async(dispatch_get_main_queue(), ^{
       self.olModulation.text = currentModulation;
  });
    
}



- (void)rfidOnOffTimeReceived:(NSData*)data
{
    AsReaderInfo *info = [AsReaderInfo sharedInstance];
    NSLog(@"[ONOFF] ON Time %d",info.rfidOnTime);
    NSLog(@"[ONOFF] OFF Time %d",info.rfidOffTime);
    NSLog(@"[ONOFF] nCst %d",info.carrierSenseTime);
    NSLog(@"[ONOFF] nRfl %d",info.rfidListenBeforeTalk);
    NSLog(@"[ONOFF] nLbt %d",info.rfidListenBeforeTalk);
    NSLog(@"[ONOFF] nCw %d",info.rfidContinuousWave);
    NSLog(@"[ONOFF] PH %d",info.rfidFrequencyHopping);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        self.olOnOffTime.text = [NSString stringWithFormat:@"%d/%d", info.rfidOnTime, info.rfidOffTime];
   });
}

- (void)channelReceived:(int)channel channelOffset:(int)channelOffset
{
    dispatch_async(dispatch_get_main_queue(),^{
        AsReaderInfo *info = [AsReaderInfo sharedInstance];
        [ModelSettings sharedInstance].m_RFIDDeviceInfo.m_nRFID_RF_CHANNEL = info.nRFIDchannel;
        [self.olRFChannel setText:[NSString stringWithFormat:@"%d",info.nRFIDchannel]];
    });
}

- (void)txPowerLevelReceived:(NSData *)power
{
    AsReaderInfo *info = [AsReaderInfo sharedInstance];
    dispatch_async(dispatch_get_main_queue(), ^{
        self.olPowerLever.text = [NSString stringWithFormat:@"%02.1f", info.rfidPower/10.0];
    });
}

- (void)anticolParamReceived:(int)mode Counter:(int)counter
{
    [ModelSettings sharedInstance].m_RFIDDeviceInfo.m_nAntiMode = mode;
    dispatch_async(dispatch_get_main_queue(), ^{
        self.olAntiCollisionMode.text = [NSString stringWithFormat:@"Mode :%d", [ModelSettings sharedInstance].m_RFIDDeviceInfo.m_nAntiMode];
    });
}

- (void)plugged:(BOOL)plug {
    
}

- (void)readerConnected:(int)status {
    
}

@end
