//
//  SensorViewController.m
//  AsReader 6Combo
//
//  Created by sps on 2020/11/12.
//

#import "SensorViewController.h"
#import "GlobalDefine.h"
#import "ModelSettings.h"
#import "RFMManager.h"
#import "ModelSettings.h"
#import "RFMSensorItem.h"
#import "Utils.h"
#import "RFMItemCell.h"

#import <AsReaderDockSDK/AsReaderDockSDK.h>

@interface SensorViewController ()<UITabBarDelegate, UITableViewDataSource, AsReaderRFIDDeviceDelegate>{
    int nTagCount;
    int nScanTime ;
    int nCycle ;
    NSMutableArray *arrayItems;
    RFMManager *rfmManager;
}

@property (weak, nonatomic) IBOutlet UIButton *btnStartRead;
@property (weak, nonatomic) IBOutlet UIButton *btnClear;
@property (weak, nonatomic) IBOutlet UIButton *btnStop;
@property (weak, nonatomic) IBOutlet UITextField *txtFWet;
@property (weak, nonatomic) IBOutlet UITextField *txtFRSSI;
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@end

@implementation SensorViewController

#define AllTag 0x00
#define S3 0x03
#define S2 0x02

#pragma mark - Lifecycle
//----------------------------------------------------------------
- (void)viewDidLoad
{
    [super viewDidLoad];
    rfmManager = [RFMManager new];
    arrayItems = [NSMutableArray new];
    self.tableView.rowHeight = 100;
    UITapGestureRecognizer *gestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(hideKeyboard)];
    gestureRecognizer.numberOfTapsRequired = 1;
    gestureRecognizer.cancelsTouchesInView = NO;
    [_tableView addGestureRecognizer:gestureRecognizer];
}

-(void)hideKeyboard{
    [self.view endEditing:YES];
}
- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [AsReaderRFIDDevice sharedInstance].delegateRFID = self;
    [[AsReaderRFIDDevice sharedInstance] getRFIDModuleVersion];
    
    [self.btnStartRead setEnabled:YES];
    [self.btnClear setEnabled:NO];
    [self.btnStop setEnabled:NO];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [[AsReaderRFIDDevice sharedInstance] stopScan];
    [AsReaderRFIDDevice sharedInstance].delegateRFID = nil;
}

#pragma mark - IBAction
//----------------------------------------------------------------
- (IBAction)clearBtnAction:(id)sender
{
    [arrayItems removeAllObjects];
    [self.tableView reloadData];
}

- (IBAction)stopBtnAction:(id)sender
{
    [[AsReaderRFIDDevice sharedInstance] stopScan];
    [self.btnStop setEnabled:NO];
    [self.btnStartRead setEnabled:YES];
    [self.btnClear setEnabled:YES];
}

- (IBAction)startBtnAction:(id)sender
{
    int rssi = [[_txtFRSSI text]intValue];
    int wet = [[_txtFWet text]intValue];
    
    if(rssi<0||rssi>31) {
        [Utils showAlert:@"" message:@"Check the input range RSSI : (0~31)" view:self];
        return;
    }
    [rfmManager setFilterRSSI:rssi wet:wet];
    
    [_btnStartRead setEnabled:NO];
    [_btnClear setEnabled:NO];
    [_btnStop setEnabled:YES];
    [[AsReaderRFIDDevice sharedInstance] startReadTagsRFM:AllTag maxTags:nTagCount maxTime:nScanTime repeatCycle:nCycle];
}

#pragma mark - AsReaderRFIDDeviceDelegate
//----------------------------------------------------------------
- (void)pcEpcSensorDataReceived:(NSData *)pcEpc sensorData:(NSData *)sensorData rssi:(int8_t)rssi
{
    RFMSensorItem *item = [rfmManager parseData:pcEpc sensorData:sensorData rssi:rssi];
    if(item ==nil)
        return;
    
    BOOL isNew = YES;
    for(int i=0; i<arrayItems.count; i++){
        RFMSensorItem *oldItem = [arrayItems objectAtIndex:i];
       
        if([item.pcEpc isEqual:oldItem.pcEpc]){
            oldItem.sensorData = item.sensorData;
            oldItem.cop = item.cop;
            oldItem.cfq = item.cfq;
            oldItem.ct = item.ct;
            oldItem.cd = item.cd;
            oldItem.temperature = item.temperature;
            oldItem.moisture = item.moisture;
            oldItem.moistureResult = item.moistureResult;
            oldItem.rssi = item.rssi;
            oldItem.count = oldItem.count+1;
            NSLog(@" ==> compare %@  %@  %d",item.pcEpc,oldItem.pcEpc,oldItem.count);
            [arrayItems replaceObjectAtIndex:i withObject:oldItem];
            isNew = NO;
            break;
        }
    }
    if(isNew){
        item.count = item.count+1;
        [arrayItems addObject:item];
    }
    NSLog(@" ==> count %d",(int)[arrayItems count]);
    [self.tableView reloadData];
}

#pragma mark - UITableView datasource
//----------------------------------------------------------------
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [arrayItems count];
}

#pragma mark - UITableView delegate
//----------------------------------------------------------------
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *identifier = @"RFMCell";

    RFMItemCell *cell = [tableView dequeueReusableCellWithIdentifier:identifier forIndexPath:indexPath];
    
    int row = (int)[indexPath row];
    RFMSensorItem *item = [arrayItems objectAtIndex:row];
    
    cell.labelEPC.text = item.pcEpc;
    cell.labelMois.text = [NSString stringWithFormat:@"%@ [%@]",item.moisture,item.moistureResult];
    cell.labelTemp.text = item.temperature;
    cell.labelEtc.text = [NSString stringWithFormat:@"%@ %@ %@  [RSSI:%@]",item.ct,item.cfq,item.cop,item.rssi];
    cell.labelCount.text = [NSString stringWithFormat:@"%d",item.count];
    return cell;
}

@end

